@@ -12,17 +12,21 @@
 #' @export
 sample_imp <- function(variable){
 
+  if(is.data.frame(variable)){
+    stop("You passed a data.frame instead of a vector to sample_imp.")
+  }
   if(all(is.na(variable))) stop("Variable consists only of NAs.")
   ret <- variable
-  ret[is.na(ret)] <- sample(size = sum(is.na(variable)),
-                             variable[!is.na(variable)], replace = TRUE)
+  need_replacement <- is.na(ret) | is.infinite(ret)
+  ret[need_replacement] <- sample(size = sum(need_replacement),
+                             variable[!need_replacement], replace = TRUE)
   return(ret)
 }
 
 
 #' Get the type of variables.
 #'
-#' Function checks wether a variable is: ...
+#' Function checks whether a variable is: ...
 #' \itemize{
 #'  \item continuous (numeric values),
 #'  \item semicontinuous (numeric values with more than 5\% of them are 0),
@@ -32,7 +36,7 @@ sample_imp <- function(variable){
 #'  \item an intercept (the same value for all observations),
 #'  \item binary (two different values - like 0s and 1s or "m" and "f"),
 #'  \item categorical (the variable is a factor or has more than 3 different values)
-#'  \item orderd categorical (the categorical variable is ordered.)
+#'  \item ordered categorical (the categorical variable is ordered.)
 #'}
 #'
 #' @param variable A variable (vector) from your data set.
@@ -143,8 +147,8 @@ get_type <- function(variable){
 #'
 #' In \code{hmi} the user can add a list of types. This function gives him a framework
 #' with suggestions. Of course he can changes on his own afterwards.
-#' For example, if a continouos variable as only two observations left, then get_type
-#' interprete this as a binary variable and not a continouos.
+#' For example, if a continuous variable as only two observations left, then get_type
+#' interprete this as a binary variable and not a continuous.
 #' @param data the data.frame also passed to \code{hmi}.
 #' @return a list with suggested types. Each list element has the name of a variable
 #' in the data.frame. The elements contain a single character denoting the type of the variable.
@@ -164,7 +168,7 @@ list_of_types_maker <- function(data){
 #' Then the results are pooled by simply averaging the results. So the user has to make sure that
 #' his analysis produces results with a meaningful average. And furthermore has to accept that no
 #' variance is calculated for these parameters.
-#' @param mids A \code{mids} (multiple imputed data set) object.
+#' @param mids A \code{mids} (multiply imputed data set) object.
 #' Either from the \code{hmi} imputation function or \code{mice}.
 #' @param analysis_function A user generated function that contains the model and the model parameters
 #' he is interested in. See examples.
@@ -234,13 +238,13 @@ hmi_pool <- function(mids, analysis_function){
 #' This function based on Drechsler, Kiesl & Speidel (2015)
 #' is needed in the imputation routine for rounded income.
 #' It calculates the likelihood contribution of the data
-#' (regardles whether they are observed precisly or presumably rounded).
+#' (regardless whether they are observed precisely or presumably rounded).
 #' @param para This is the vector \eqn{Psi} of parameters
 #' (see p. 62 in Drechsler, Kiesl & Speidel, 2015).
 #' With respect to them, the value returned by negloglik2 shall be
 #' maximized.\cr
 #' The starting values are c(kstart, betastart2, gammastart, sigmastart2)
-#' (the 6 treshholds (or "cutting points") for the latent variable behind the rounding degree,
+#' (the 6 thresholds (or "cutting points") for the latent variable behind the rounding degree,
 #' the regression parameters explaining the logged income,
 #' the regression parameters explaining the rounding degree
 #' and the variance parameter).
@@ -248,7 +252,7 @@ hmi_pool <- function(mids, analysis_function){
 #' @param y_in_negloglik the target variable (a vector).
 #' @param lower the lower bound of an interval variable.
 #' @param upper the upper bound of an interval variable.
-#' @param my_p This vector is the indicator of the (highes possible) rounding degree for an observation.
+#' @param my_p This vector is the indicator of the (highest possible) rounding degree for an observation.
 #' This parameter comes directly from the data.
 #' @param mean_y_precise the scalar with the value of the mean of the target variable.
 #' @param sd_y_precise the scalar with the value equal to the standard deviation of the target variable.
@@ -262,7 +266,7 @@ negloglik2 <- function(para, X, y_in_negloglik, lower = NA, upper = NA,
   lower <- ifelse(is.na(lower), -Inf, lower)
   upper <- ifelse(is.na(upper), Inf, upper)
 
-  # the first 6 parameters are the tresholds for the rounding degree
+  # the first 6 parameters are the thresholds for the rounding degree
   # below k0: no rounding, between k0 and k1: rounding degree 5 etc.
   # The other degrees are 10, 50, 100, 500 and 1000
   k0 <- para[1]
@@ -314,10 +318,10 @@ negloglik2 <- function(para, X, y_in_negloglik, lower = NA, upper = NA,
   y_obs <- y_in_negloglik[!missind_negloglik]
 
   #get the likelihood contributions of the imprecise observations
-  intervall_obs <- !is.na(lower)
-  sum_likelihood_imprecise_obs <- sum(log(contributions4intervalls(lower = lower[intervall_obs],
-                                                                   upper = upper[intervall_obs],
-                                                                   mymean = mean2[intervall_obs],
+  interval_obs <- !is.na(lower)
+  sum_likelihood_imprecise_obs <- sum(log(contributions4intervals(lower = lower[interval_obs],
+                                                                   upper = upper[interval_obs],
+                                                                   mymean = mean2[interval_obs],
                                                                    mysd = sigma2)))
 
   # a1 are a7 the "components" of the log-liklihood contributions
@@ -326,7 +330,7 @@ negloglik2 <- function(para, X, y_in_negloglik, lower = NA, upper = NA,
   # if the income is divisable by 5, but not by 10, a1 + a2 are relevant
   # etc.
 
-  #get the value of the standard bivariate normal cumulative density funtion
+  #get the value of the standard bivariate normal cumulative density function
   #the better k0, k1, ..., sigma and rho are found, the better a1, a2, ...
 
   a1 <- pbivnormX(x =  c(k0 - mean1[!missind_negloglik])/sigma1,
@@ -430,9 +434,9 @@ negloglik2 <- function(para, X, y_in_negloglik, lower = NA, upper = NA,
   return(-ret) # notice the minus
 }
 
-#' calculate the likelihood contribution of the intervall data only
+#' calculate the likelihood contribution of the interval data only
 #'
-#' calculate the likelihood contribution of the intervall data only
+#' calculate the likelihood contribution of the interval data only
 #' @param para This is the vector \eqn{Psi} of parameters
 #' (see p. 62 in Drechsler, Kiesl & Speidel, 2015).
 #' With respect to them, the value returned by negloglik2 shall be
@@ -473,10 +477,10 @@ negloglik2_intervalsonly <- function(para, X, lower, upper){
 
 
   #get the likelihood contributions of the imprecise observations
-  intervall_obs <- !is.na(lower)
-  sum_likelihood_imprecise_obs <- sum(log(contributions4intervalls(lower = lower[intervall_obs],
-                                                                   upper = upper[intervall_obs],
-                                                                   mymean = mean2[intervall_obs],
+  interval_obs <- !is.na(lower)
+  sum_likelihood_imprecise_obs <- sum(log(contributions4intervals(lower = lower[interval_obs],
+                                                                   upper = upper[interval_obs],
+                                                                   mymean = mean2[interval_obs],
                                                                    mysd = sigma2)))
   return(-sum_likelihood_imprecise_obs) # notice the minus
 }
@@ -489,11 +493,11 @@ negloglik2_intervalsonly <- function(para, X, lower, upper){
 #' @param mymean a numeric for the expected value of the normal distribution
 #' (which is one of the parameters trying to be optimized
 #' so that the likelihood becomes maximized)
-#' @param mysd a numeric for the standard devitation of the normal distribution
+#' @param mysd a numeric for the standard deviation of the normal distribution
 #' (which is one of the parameters trying to be optimized
 #' so that the likelihood becomes maximized)
 #' @return a vector giving the likelihood contributions of the interval data.
-contributions4intervalls <- function(lower, upper, mymean, mysd){
+contributions4intervals <- function(lower, upper, mymean, mysd){
 
   ret <- stats::pnorm(upper, mean = mymean, sd = mysd) -
     stats::pnorm(lower, mean = mymean, sd = mysd)
@@ -536,12 +540,12 @@ pbivnormX <- function (x, y, rho = 0) {
 }
 
 
-#' Function to get the likelihood contribution of different roudinding degrees
+#' Function to get the likelihood contribution of different rounding degrees
 #'
 #' It is needed in the imputation routine for rounded income.
 #' See equation (5) in Drechsler, Kiesel & Speidel (2015)
-#' @param ki An integer for the i-th treshold (or "cutpoint")
-#' @param kj An integer for the j-th treshold (or "cutpoint") (ki < kj)
+#' @param ki An integer for the i-th threshold (or "cutpoint")
+#' @param kj An integer for the j-th threshold (or "cutpoint") (ki < kj)
 #' @param mean1_obs A vector with the expected value of G for the observed data
 #' @param mean2_obs A vector with the expected value of log(Y) for the observed data
 #' @param sigma1 A scalar for the variance of the G
@@ -573,7 +577,7 @@ components <- function(ki, kj, mean1_obs, mean2_obs, sigma1, sigma2, rho, y_obs,
 
 #' Function to calculate double integrals
 #'
-#' This function is primarily build to make the funtion \code{components} more neat.
+#' This function is primarily build to make the function \code{components} neater.
 #' @param lower_inner The vector for the lower bound for the inner integral
 #' @param upper_inner The vector for the upper bound for the inner integral
 #' @param lower_outer The vector for the lower bound for the outer integral
@@ -605,7 +609,7 @@ doubleintegral <- function(lower_inner, upper_inner, lower_outer, upper_outer,
 #' @param variable_names_in_data A character-vector with the column names of the data set.
 #' @return A list with the names of the target variable, the intercept variable,
 #' the fixed and random effects covariates and the cluster id variable.\cr
-#' If some of them doesn't exist, they get the value "".
+#' If some of them don't exist, they get the value "".
 #' @export
 extract_varnames <- function(model_formula = NULL, constant_variables,
                              variable_names_in_data){
@@ -814,9 +818,9 @@ as_interval <- function(lower, upper){
 }
 
 
-#' Function to check wheter an object is an interval
+#' Function to check whether an object is an interval
 #'
-#' If there are numerics seperated by semicolons (;), this is considered to be an interval.
+#' If there are numerics separated by semicolons (;), this is considered to be an interval.
 #' intervals with 2.4e5 are not considered to be an interval.
 #' @param x an object
 #' @return a boolean value indicting whether x is an interval or not
@@ -918,11 +922,11 @@ sna_interval <- function(x){
   return(x)
 }
 
-#' Transform interval variables to a interval data frame
+#' Transform interval variables to an interval data frame
 #'
 #' This function is the path from this hmi package to the linLIR package (Wiencierz, 2012).
 #' @param interval an \code{interval}
-#' @return an interval data frame (idf-object) with one variable (having a lower and a upper bound).
+#' @return an interval data frame (idf-object) with one variable (having a lower and an upper bound).
 #' @export
 interval2idf <- function(interval){
   #lapply(interval, split_interval)
@@ -954,24 +958,27 @@ idf2interval <- function(idf){
 #' @param x an single element or vector to add to the interval object
 #' @return an interval object
 #' @export
-'plus_interval' <- function(interval, x){
+#' @rdname interval-add
+"+.interval" <- function(interval, x){
   tmp <- split_interval(interval) + split_interval(x)
   as_interval(lower = tmp[, 1], upper = tmp[, 2])
 }
 
 
-#' Subraction function
+#' Subtraction function
 #'
 #' Function to subtract single elements or vectors (of correct dimension) from the interval object
 #' @param interval an object from class interval
 #' @param x an single element or vector to subtract to the interval object
 #' @return an interval object
 #' @export
-'minus_interval' <- function(interval, x){
+#' @rdname interval-subtract
+"-.interval" <- function(interval, x){
   tmp <- split_interval(interval) - split_interval(x)
   as_interval(lower = tmp[, 1], upper = tmp[, 2])
 }
 
+
 #' Multiplication function
 #'
 #' Function to multiply single elements or vectors (of correct dimension) to the interval object
@@ -979,7 +986,8 @@ idf2interval <- function(idf){
 #' @param x an single element or vector for multplication
 #' @return an interval object
 #' @export
-'times_interval' <- function(interval, x){
+#' @rdname interval-multiply
+"*.interval" <- function(interval, x){
   tmp <- split_interval(interval) * split_interval(x)
   as_interval(lower = tmp[, 1], upper = tmp[, 2])
 }
@@ -991,9 +999,71 @@ idf2interval <- function(idf){
 #' @param x an single element or vector for division
 #' @return an interval object
 #' @export
-'divided_interval' <- function(interval, x){
+#' @rdname interval-divide
+"/.interval" <- function(interval, x){
   tmp <- split_interval(interval) / split_interval(x)
   as_interval(lower = tmp[, 1], upper = tmp[, 2])
 }
 
 
+#' Standardizing function
+#'
+#' Function to standardize variables that are numeric (continuous and count variables) but no rounded continuous, semicontinuous, intercepts or categorical variables.
+#' @param X A n times p data.frame with p fixed (or random) effects variables.
+#' @return A n times p data.frame with the standardized versions of the numeric variables.
+#' @export
+stand <- function(X){
+  if(!is.data.frame(X)) stop("X has to be a data.frame.")
+  need_stand_X <- apply(X, 2, get_type) %in% c("cont", "count")
+  X_stand <- X
+  X_stand[, need_stand_X] <- scale(X[, need_stand_X])
+  return(X_stand)
+}
+
+#' Removing excessive factors
+#'
+#' Function to exclude variables that have too many levels as they may cause numerical problems
+#' @param X A n times p data.frame with p fixed (or random) effects variables.
+#' @param k An integer defining the allowed maximum of levels in a factor covariate.
+#' @return A n times (p-r) data.frame, with r beeing the number of variables with too many factors.
+#' @export
+remove_excessives <- function(X, k = 10){
+
+  if(!is.data.frame(X)) stop("X has to be a data.frame.")
+
+  if(!is.numeric(k)) stop("k has to be a single numeric value.")
+  if(length(k) > 1)  stop("k has to be a single numeric value.")
+
+  # work out the type of the variables
+  types <- array(dim = ncol(X))
+  for(j in 1:length(types)) types[j] <- get_type(X[, j])
+
+  # currently, R interpretes interval data as categorical.
+  # This can be unfeasible as there could be as many categories as observations.
+  # Two practical approaches are 1. ignoring interval data. 2. splitting them up into
+  # two separate variables. Currently we chose option 2.
+  interval <- types == "interval"
+  added_variables <- as.data.frame(matrix(NA, nrow = nrow(X), ncol = 2 * sum(interval)))
+  interval_counter <- 1
+  for(l in which(interval)){
+    lower <- sample_imp(split_interval(X[, l])[, 1])
+    upper <- sample_imp(split_interval(X[, l])[, 2])
+    added_variables[, 2 * (interval_counter - 1) + 1:2] <- cbind(lower, upper)
+
+    interval_counter <- interval_counter + 1
+  }
+
+  # determine the "categorical" variables.
+  categorical <- types == "categorical"
+
+  # get the name of the variables, that have too many levels
+  too_many_levels <- colnames(X[, categorical, drop = FALSE])[
+    apply(X[, categorical, drop = FALSE], 2, function(x) nlevels(factor(x))) > k]
+
+  tmp <- X[, !names(X) %in% too_many_levels & !interval, drop = FALSE]
+
+  ret <- cbind(tmp, added_variables)
+  if(ncol(ret) == 0) stop("No variables left after removing excessive variables.")
+
+  return(ret)
+}
